﻿/**
    autorg_kek_client.cs

    Copyright (c) 2016, Masatsuyo Takahashi, KEK-PF
    
    thanks to:
    http://www.codeproject.com/Articles/1415/Introduction-to-TCP-client-server-in-C
 */
using System;
using System.IO;
using System.Net;
using System.Text;
using System.Net.Sockets;
using System.Collections.Generic;
using System.Diagnostics;
using System.Reflection;
using System.Threading;
using System.Text.RegularExpressions;

public class clnt
{
    static bool DEBUG = false;
    const string HELP_MESSAGE =
        "Usage: autorg_kek[options] <datafile(s)>\n" +
        "Options:\n" +
        "  -h, --help show this help message and exit" +
        "  --mininterval=MININTERVAL\n" +
        "                        Minimum acceptable Guinier interval length in points.\n" +
        "                       (default: 3)\n" +
        "  --smaxrg=SMAXRG Maximum acceptable SmaxRg value. (default: 1.3)\n" +
        "  --sminrg=SMINRG Minimum acceptable SminRg value. (default unlimited)\n" +
        "  -o OUTPUT, --output=OUTPUT\n" +
        "                        Relative or absolute path to save result\n" +
        "  -f FORMAT, --format=FORMAT\n" +
        "                        Output format, either of: csv, ssv\n" +
        "  -A, --ATSAS           output ATSAS autorg columns only\n" +
        "  -r, --robust          output result even if it is quite bad\n" +
        "  -S, --start_server    start server\n" +
        "  -P, --stop_server     stop server\n" +
        "  -v, --version show the appication's version info\n"
        ;
    const string HOST = "localhost";
    const Int32 PORT = 49999;
    const string START_SERVER = "--start_server";
    const string ASK_READY = "ASK_READY";
    const string STOP_SERVER = "--stop_server";
    const string EXECUTE = "EXECUTE";
    const string OUTPUT = "--output";
    const string ATSAS = "--ATSAS";
    const string ROBUST = "--robust";
    const string HELP = "--help";
    const string VERSION = "--version";
    const string SUCCESS = "SUCCESS";
    const string MESSAGE = "MESSAGE";
    const string ACK = "ACK";

    //--------------------------------------------------------------------------
    //  Request
    //--------------------------------------------------------------------------
    public static List<string> Request(ref List<string> args, bool silent = true)
    {
        List<string> reply = new List<string>();
        try
        {
            TcpClient tcpclnt = new TcpClient();
            if (DEBUG)
            {
                Console.WriteLine("Connecting.....");
            }

            tcpclnt.Connect(HOST, PORT);

            /*
            if (tcpclnt.ConnectAsync(HOST, PORT).Wait(1000))
            {
                if (DEBUG)
                {
                    Console.WriteLine("Connected");
                }
            }
            else
            {
                // connection failure
                throw new Exception("Failed to connect.");
            }
            */

            // use the ipaddress as in the server program
            Stream stm = tcpclnt.GetStream();

            ASCIIEncoding asen = new ASCIIEncoding();

            string ret = "";
            foreach (string arg in args)
            {
                if (DEBUG)
                {
                    Console.WriteLine("Sending " + arg);
                }
                byte[] ba = asen.GetBytes(arg);

                stm.Write(ba, 0, ba.Length);

                byte[] bb = new byte[1024];
                int k = stm.Read(bb, 0, 1024);

                ret = System.Text.Encoding.UTF8.GetString(bb, 0, k);
                if (DEBUG)
                {
                    Console.WriteLine(ret);
                }
            }
            tcpclnt.Close();
            if (ret == SUCCESS || ret == ACK)
            {
                Environment.ExitCode = 0;
                reply.Add(ret);
            }
            else
            {
                string pattern = MESSAGE + ";(.*)";
                string replacement = "$1";
                Regex rgx = new Regex(pattern);
                string result = rgx.Replace(ret, replacement);
                if (DEBUG)
                {
                    Console.WriteLine("Original String:    '{0}'", ret);
                    Console.WriteLine("Replacement String: '{0}'", result);
                }
                if (result == ret)
                {
                    Environment.ExitCode = 1;
                    reply.Add(ret);
                }
                else
                {
                    reply.Add(result);
                    Environment.ExitCode = 0;
                }
            }
        }

        catch (Exception e)
        {
            if (!silent)
            {
                Console.WriteLine("Error..... " + e.StackTrace);
            }
        }

        return reply;
    }

    //--------------------------------------------------------------------------
    //  StartServer
    //--------------------------------------------------------------------------
    public static void StartServer()
    {
        try
        {
            // set current directory to the disttribution home
            Assembly assem = typeof(clnt).Assembly;
            string work_dir = Path.GetDirectoryName(Path.GetDirectoryName(assem.Location));
            System.IO.Directory.SetCurrentDirectory(work_dir);
            if (DEBUG)
            {
                System.Console.WriteLine("current directory={0} \n", work_dir);
            }

            // run autorg_kek.exe as a server
            ProcessStartInfo startInfo = new ProcessStartInfo(@"executables\autorg_kek.exe");
            startInfo.Arguments = "-S";
            startInfo.UseShellExecute = false;
            Process process = Process.Start(startInfo);

            // wait until the server is ready
            List<string> args = new List<string>();
            args.Add(ASK_READY);
            bool server_ready = false;
            int num_trials = 0;
            while (!server_ready)
            {
                if (num_trials++ > 60)
                {
                    throw new Exception("Server did'nt reply in an expected time!");
                    break;
                }

                try
                {
                    List<string> reply = Request(ref args, true);
                    if (reply[0] == ACK)
                    {
                        server_ready = true;
                    }
                }
                catch (Exception e_)
                {
                    if (DEBUG)
                    {
                        Console.WriteLine("Not Ready:" + e_.StackTrace);
                    }
                }
                Thread.Sleep(1000);
            }
        }
        catch (Exception e)
        {
            Console.WriteLine("Error..... " + e.StackTrace);
            Environment.ExitCode = 1;
        }
    }

    //--------------------------------------------------------------------------
    //  StopServer
    //--------------------------------------------------------------------------
    public static void StopServer()
    {
        List<string> args = new List<string>();
        args.Add(STOP_SERVER);
        Request(ref args);
    }

    //--------------------------------------------------------------------------
    //  ShowHelp
    //--------------------------------------------------------------------------
    public static void ShowHelp()
    {
        System.Console.Write(HELP_MESSAGE);
    }

    //--------------------------------------------------------------------------
    //  ShowVersion
    //--------------------------------------------------------------------------
    public static void ShowVersion()
    {
        List<string> args = new List<string>();
        args.Add(VERSION);
        List<string> reply = Request(ref args);
        if (reply.Count > 0)
        {
            System.Console.WriteLine(reply[0]);
            Environment.ExitCode = 0;
        }
        else
        {
            System.Console.WriteLine("Can't connect to the autorg_kek server!");
            Environment.ExitCode = 1;
        }
    }

    //--------------------------------------------------------------------------
    //  Execute
    //--------------------------------------------------------------------------
    public static void Execute(ref List<string> input_files, string output_file, bool atsas, bool robust)
    {
        // assert( input_files.Count == 1 )

        List<string> args = new List<string>(input_files);
        // args.Insert(0,output_file);
        args.Add(output_file);

        foreach (string arg in args)
        {
            // TBD: relative path to absolute path
        }

        if (atsas)
        {
            args.Add(ATSAS);
        }
        if (robust)
        {
            args.Add(ROBUST);
        }
        args.Add(EXECUTE);

        Request(ref args);
    }

    //--------------------------------------------------------------------------
    //  Main
    //--------------------------------------------------------------------------
    public static void Main(string[] args)
    {
        /**
            arg[0] : in_file
            arg[1] : out_file
            arg[2] : ATSAS  0 or 1
            arg[3] : rubust 0 or 1
         */
        string env = System.Environment.GetEnvironmentVariable("AUTORG_KEK_CLIENT_DEBUG");
        if (env == "1")
        {
            DEBUG = true;
            System.Console.WriteLine("autorg_kek_client started in debugging mode.");
        }
        bool start_server = false;
        bool stop_server = false;
        bool atsas = false;
        bool robust = false;
        List<string> input_files = new List<string>();
        string output_file = "";
        int output_file_index = -1;

        if (args.Length == 0)
        {
            ShowHelp();
        }

        for (int i = 0; i < args.Length; ++i)
        {
            if (DEBUG)
            {
                System.Console.WriteLine("args[{0}]={1} \n", i, args[i]);
            }
            string arg = args[i];
            if (arg == "-S" || arg == START_SERVER)
            {
                start_server = true;
            }
            else if (arg == "-P" || arg == STOP_SERVER)
            {
                stop_server = true;
            }
            else if (arg == "-o" || arg == OUTPUT)
            {
                if (i < args.Length - 1)
                {
                    output_file_index = i + 1;
                    output_file = args[output_file_index];
                }
            }
            else if (arg == "-A" || arg == ATSAS)
            {
                atsas = true;
            }
            else if (arg == "-r" || arg == ROBUST)
            {
                robust = true;
            }
            else if (arg == "-h" || arg == HELP)
            {
                ShowHelp();
            }
            else if (arg == "-v" || arg == VERSION)
            {
                ShowVersion();
            }
            else
            {
                if (output_file_index < 0 || i != output_file_index)
                {
                    // System.Console.WriteLine("Input file: {0}", arg);
                    input_files.Add(arg);
                }
            }
        }

        if (start_server || stop_server)
        {
            if (args.Length != 1)
            {
                // TBD
            }
            if (start_server)
            {
                StartServer();
            }
            else
            {
                StopServer();
            }
        }
        else
        {
            if (input_files.Count > 0)
            {
                Execute(ref input_files, output_file, atsas, robust);
            }
        }
    }

}