﻿/**
    cshap_tester.cs

    Copyright (c) 2016, Masatsuyo Takahashi, KEK-PF
 */
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Runtime.InteropServices;

namespace csharp_test
{
    /**
        thanks to:

        Relative path to unmanaged DLL [duplicate]
        http://stackoverflow.com/questions/9256901/relative-path-to-unmanaged-dll

        kernel32.GetDllDirectory returns error (0) when called via pinvoke
        http://stackoverflow.com/questions/19822156/kernel32-getdlldirectory-returns-error-0-when-called-via-pinvoke
     */

    public class UnsafeNativeMethods
    {
        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern bool SetDllDirectory(string lpPathName);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern int GetDllDirectory(int bufsize, StringBuilder buf);

        [DllImport("kernel32", CharSet = CharSet.Unicode, SetLastError = true)]
        public static extern IntPtr LoadLibrary(string librayName);

        [DllImport("kernel32.dll", SetLastError = true)]
        public static extern int GetCurrentDirectory(int bufsize, StringBuilder buf);
    }

    class DllLoader
    {
        public static void Load(string dllfolder, string libname)
        {
            var currentpath = new StringBuilder(255);
            UnsafeNativeMethods.GetDllDirectory(currentpath.Length, currentpath);

            // use new path
            UnsafeNativeMethods.SetDllDirectory(dllfolder);

            UnsafeNativeMethods.LoadLibrary(libname);

            // restore old path
            UnsafeNativeMethods.SetDllDirectory(currentpath.ToString());
        }
    }

    class Program
    {
        [DllImport("autorg_kek_glue.dll", CallingConvention = CallingConvention.Cdecl)]
        private extern static int Autorg_Initialize(char[] autorg_lib_dir);

        [DllImport("autorg_kek_glue.dll", CallingConvention = CallingConvention.Cdecl)]
        private extern static int Autorg_Execute(char[] data_file, char[] result_file, int ATSAS, int robust);

        [DllImport("autorg_kek_glue.dll", CallingConvention = CallingConvention.Cdecl)]
        private extern static int Autorg_Finalize();

        static void Main(string[] args)
        {
            for (int i = 0; i < args.Length; ++i)
            {
                System.Console.WriteLine("args[{0}]={1} \n", i, args[i]);
            }

            // Load the glue dll
            DllLoader.Load(args[0], "autorg_kek_glue");

            int ret;

            ret = Autorg_Initialize(args[1].ToCharArray());
            // ret = Autorg_Initialize( "F:/Guinier/autorg/autorg_kek-0_3_4-x64/lib".ToCharArray() );
            System.Console.WriteLine("Autorg_Initialize: ret={0} \n", ret.ToString());

            ret = Autorg_Execute(args[2].ToCharArray(), args[3].ToCharArray(), Convert.ToInt32(args[4]), Convert.ToInt32(args[5]) );
            System.Console.WriteLine("Autorg_Execute: ret={0} \n", ret.ToString());

            Autorg_Finalize();
        }
    }
}
