﻿using System;
using System.IO;
using System.Collections.Generic;
using System.Text;
using System.Threading.Tasks;
using System.Net.Sockets;

namespace serial_analyzer_client_example
{
    class serial_analyzer_client_example
    {
        const string HOST = "localhost";
        const Int32 PORT = 49999;
        const string SERIAL_ANALYZER = "SERIAL_ANALYZER";
        const string PROCESS_ARGS = "PROCESS_ARGS";
        const string START_PROCESS = "START_PROCESS";
        const string GET_NUM_STEPS = "GET_NUM_STEPS";
        const string REQUEST_INFO = "REQUEST_INFO";
        const string CANCEL_PROCESS = "CANCEL_PROCESS";

        static ASCIIEncoding ascii_encoding = new ASCIIEncoding();

        static TcpClient client;
        static Stream stream;

        // 送信
        static string Send(string arg)
        {
            Console.WriteLine("Sending " + arg);
            byte[] ba = ascii_encoding.GetBytes(arg);
            stream.Write(ba, 0, ba.Length);

            byte[] bb = new byte[1024];
            int k = stream.Read(bb, 0, 1024);
            string ret = System.Text.Encoding.UTF8.GetString(bb, 0, k);
            Console.WriteLine(ret);
            return ret;
        }

        // SerialAnalyzer実行のためのサーバ接続
        static void connectSerialAnalyzer()
        {
            client = new TcpClient();
            client.Connect(HOST, PORT);
            stream = client.GetStream();
            Send(SERIAL_ANALYZER);
        }

        // SerialAnalyzerの開始
        static int startSerialAnalyzer(ref List<string> args)
        {
            // 引数方針通知
            Send(PROCESS_ARGS);

            foreach (string arg in args)
            {
                Send(arg);
            }

            // 処理開始指示
            Send(START_PROCESS);
            string ret = Send(GET_NUM_STEPS);

            return Convert.ToInt32(ret);
        }

        // SerialAnalyzerの中止
        static void cancelSerialAnalyzer()
        {
            Send(CANCEL_PROCESS);
        }

        // SerialAnalyzerからの情報取得
        static int getSerialAnalyzerInfo()
        {
            string ret = Send(REQUEST_INFO);
            return Convert.ToInt32(ret);
        }

        // 切断
        static void disconnectSerialAnalyzer()
        {
            client.Close();
        }

        // 実行例
        static void Main(string[] args)
        {
            // 接続
            connectSerialAnalyzer();

            List<string> list = new List<string>();
            list.Add("F:/Guinier/Data/20161006/OA01/Backsub");
            list.Add("F:/Guinier/Data/20161006/OA01/spectra");
            list.Add("F:/Guinier/autorg/autorg_kek-0_6_3-x64/test/temp");

            // 開始して取り消し
            int num_steps = startSerialAnalyzer(ref list);

            for (int i=0; i < 3; ++i)
            {
                int step_count = getSerialAnalyzerInfo();
                Console.WriteLine(step_count);
            }
            cancelSerialAnalyzer();

            // 開始して完遂
            num_steps = startSerialAnalyzer(ref list);

            for (int i = 0; i < num_steps; ++i)
            {
                int step_count = getSerialAnalyzerInfo();
                Console.WriteLine(step_count);
            }

            // 切断
            disconnectSerialAnalyzer();
        }
    }
}
