# coding: utf-8

import sys
import os
import re
import time
from nose.tools             import eq_
from subprocess             import call

this_dir = os.path.dirname( os.path.abspath( __file__ ) )
sys.path.append( this_dir + '/../lib' )
m = re.search( r'autorg_kek-(\d+)_(\d+)_(\d+)', this_dir )
ver_num = int(m.group(1))*10000 + int(m.group(2))*100 + int(m.group(3))
if ver_num > 610:
    import KekLib, AutorgKek
else:
    import CommandMain
from BasicUtils             import clear_dirs_with_retry
from CommandMain            import CommandMain

sys.path.append( this_dir + '/../..' )
from TestEnv import env_dict

def test_load():
    assert True

def test_setup():
    global argv, this_dir, run_dir
    argv = sys.argv
    this_dir    = os.path.dirname( os.path.abspath( __file__ ) )
    run_dir     = os.path.dirname( this_dir )

def do_CommandMain( folder, file, out_file, subprocess=True, atsas=False, fh=None ):
    global argv, run_dir
    path    = folder + '/' + file

    if subprocess:
        exe_name = 'autorg_kek.bat'
        atsas_arg = '-A' if atsas else ''
        ret = call( '%s/%s %s -o %s %s' % ( run_dir, exe_name, atsas_arg, out_file, path ), shell=True )
        eq_( ret, 0 )
    else:
        os.chdir( run_dir )
        sys.argv = argv + [ '-o', out_file, path ]
        command = CommandMain()
        result = command.execute( path, out_file, robust=True )
        rec = ','.join( [ folder, file ]
                        + list( map( lambda x: '%g' % x, result.quality_object.get_factors() + [ result.fit.result.aic ] ) )
                      )
        fh.write( rec + '\n' )

def test_CommandMain( num_limit=None ):
    global this_dir

    folder  = env_dict[ 'in_folder3' ] + '/aggregation_20160524'
    result_dir = os.path.dirname( os.path.abspath( __file__ ) ) + '/temp'
    clear_dirs_with_retry( [ result_dir ] )

    fh = open( this_dir + '/quality_agg.csv', 'w' )

    start_ = time.time()
    n = 0
    for root, dirs, files in os.walk( folder ):
        for file in files:
            if file.find( '.dat' ) < 0: continue
            # if name is not None and file != name: continue

            result_file = file.replace( '.dat', '.csv' )
            do_CommandMain( root, file, result_dir + '/' + result_file, fh=fh )
            n += 1
        if num_limit is not None and n >= num_limit: break

    fh.close()

    end_ = time.time()
    t_ =  end_ - start_
    print( 'it took ', t_, n, t_ / n )

def test_Minimal():
    in_folder = env_dict[ 'in_folder3' ] + '/OA'
    file = 'OA_1.0_ave_sub.dat'
    out_file = 'redult.csv'
    fh = open( this_dir + '/quality.csv', 'w' )
    do_CommandMain(in_folder, file, out_file, fh=fh)
    fh.close()

if __name__ == '__main__':
    test_setup()
    # test_CommandMain( num_limit=3 )
    # test_CommandMain()
    test_Minimal()
